//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "PhCircle.hpp"

#include "MyException.hpp"
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
PhCircle::PhCircle(Group *parent, int phGroupId, const std::string &name, const Vector2D &center, float radius) : PhysicObject(parent, phGroupId, name), m_radius(radius)
{
    SetPosition(center);
}
//------------------------------------------------------------------------------
PhCircle::~PhCircle()
{
    //dtor
}
//------------------------------------------------------------------------------
void PhCircle::SetRadius(float radius)
{
    if(radius < 0)
    {
        MY_THROW("SetRadius : negative !");
    }
}
//------------------------------------------------------------------------------
float PhCircle::GetRadius() const
{
    return m_radius;
}
//------------------------------------------------------------------------------
bool PhCircle::IsPointInside(const Vector2D &point)
{
    return ((LinePtPt(GetGlobalPosition(), point)).GetSqrSum() <= pow(GetRadius(), 2));
}
//------------------------------------------------------------------------------
/*void PhCircle::RegisterClassLua(lua_State *myLuaState)
{
    luabind::module(myLuaState)
    [
        luabind::class_<Object>("Object")

    ];
}*/
//------------------------------------------------------------------------------
void PhCircle::RenderShape(sf::RenderTarget &App) const
{
    sf::Shape circle = sf::Shape::Circle(GetGlobalPosition(), GetRadius(), Color(0, 0, 0), 1, Color(0, 0, 255));
    circle.EnableFill(false);
    App.Draw(circle);
}
//------------------------------------------------------------------------------
} // namespace En
