//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "PhysicWorld.hpp"
//------------------------------------------------------------------------------
#include "MyException.hpp"

#include <iostream>
#include <utility>
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
using namespace std;
//------------------------------------------------------------------------------
PhysicWorld::PhysicWorld()
{
    //ctor
}
//------------------------------------------------------------------------------
PhysicWorld::~PhysicWorld()
{
    if(!m_phTbl.empty())
    {
        cerr << "Warning: PhysicWorld not empty when deleted !!" << endl;
        cerr << "Left: " << m_phTbl.size() << endl;
    }
}
//------------------------------------------------------------------------------
void PhysicWorld::Attach(PhysicObject *obj)
{
    // FIXME: delete test
    vector<PhysicObject*> &tbl = m_phTbl[obj->GetPhysicGroup()];

    if(find(tbl.begin(), tbl.end(), obj) != tbl.end())
    {
        MY_THROW("Object already exists ; Name : '" << obj->GetName() << "'");
    }
    tbl.push_back(obj);
}
//------------------------------------------------------------------------------
void PhysicWorld::Detach(PhysicObject *obj)
{
    // FIXME: Accelerer la recherche avec des map de pointeur !
    std::map<int, std::vector<PhysicObject*> >::iterator mapIt = m_phTbl.find(obj->GetPhysicGroup());
    vector<PhysicObject*> &tbl = mapIt->second;

    for(std::vector<PhysicObject*>::iterator it = tbl.begin(); it != tbl.end(); it++ )
    {
        if(obj == *it)
        {
            it = tbl.erase(it) - 1;

            if(tbl.empty())
            {
                m_phTbl.erase(mapIt);
            }

            return;
        }
    }

    MY_THROW("PhysicWorld::Detach() : Object '" << obj->GetName() << "' doesn't exist !");
}
//------------------------------------------------------------------------------
void PhysicWorld::ProceedIntersection()
{
    for(std::vector<std::pair<int, int> > ::iterator it = m_phGroupTbl.begin(); it != m_phGroupTbl.end(); it++ )
    {
        ProceedIntersection(it->first, it->second);
    }
}
//------------------------------------------------------------------------------
void PhysicWorld::AddPhysicGroup(int phGroupId1, int phGroupId2)
{
    for(std::vector<std::pair<int, int> > ::iterator it = m_phGroupTbl.begin(); it != m_phGroupTbl.end(); it++ )
    {
        if((it->first == phGroupId1 && it->second == phGroupId2) || (it->first == phGroupId2 && it->second == phGroupId1))
        {
            MY_THROW("PhysicWorld::AddPhysicGroup() : This group pair alredy exist !");
        }
    }

    m_phGroupTbl.push_back(std::pair<int, int>(phGroupId1, phGroupId2));
}
//------------------------------------------------------------------------------
void PhysicWorld::ProceedIntersection(int phGroupId1, int phGroupId2)
{
    std::map<int, std::vector<PhysicObject*> >::iterator itGrp1 = m_phTbl.find(phGroupId1);
    if(itGrp1 == m_phTbl.end())
    {
        return; // Nothing to intersect
    }

    std::map<int, std::vector<PhysicObject*> >::iterator itGrp2 = m_phTbl.find(phGroupId2);
    if(itGrp2 == m_phTbl.end())
    {
        return; // Nothing to intersect
    }

    Vector2D pos(0, 0), penetrationVector(0, 0);


    if(itGrp1 == itGrp2) // Inter group => less test to do
    {
        // We create a copy, in case some object are destroyed during the events, moreover, we use protected ptrs
        std::vector<ProtectedPtr<PhysicObject> > tbl1;
        tbl1.reserve(itGrp1->second.size());
        tbl1.insert(tbl1.begin(), itGrp1->second.begin(), itGrp1->second.end());

        /*for(size_t i = 0; i < tbl1.size(); i++)
        {
            for(size_t j = i + 1; j < tbl2.size(); j++)
            {
                // penetrationVector => Pour faire sortir *tbl1[i] de *tbl2[j]
                if(tbl1[i]->Intersect(*tbl2[j], pos, penetrationVector))
                {
                    tbl1[i]->OnIntersect(tbl1[i], tbl2[j], pos, penetrationVector);
                    tbl2[j]->OnIntersect(tbl2[j], tbl1[i], pos, -penetrationVector);
                }
            }
        }*/

        for(std::vector<ProtectedPtr<PhysicObject> >::iterator it1 = tbl1.begin(); it1 != tbl1.end(); it1++)
        {
            for(std::vector<ProtectedPtr<PhysicObject> >::iterator it2 = it1 + 1; it2 != tbl1.end(); it2++)
            {
                if(it1->IsValid() && it2->IsValid()) // Not deleted yet
                {
                    PhysicObject *obj1 = it1->GetPtr();
                    PhysicObject *obj2 = it2->GetPtr();

                    // penetrationVector => Pour faire sortir *obj1 de *obj2
                    if(obj1->Intersect(*obj2, pos, penetrationVector))
                    {
                        obj1->OnIntersect(obj1, obj2, pos, penetrationVector);

                        // Might be deleted in the first event !
                        if(it1->IsValid() && it2->IsValid()) // Not deleted yet
                        {
                            obj2->OnIntersect(obj2, obj1, pos, -penetrationVector);
                        }
                    }
                }
            }
        }
    }
    else
    {
        // We create a copy, in case some object are destroyed during the events, moreover, we use protected ptrs
        std::vector<ProtectedPtr<PhysicObject> > tbl1;
        tbl1.reserve(itGrp1->second.size());
        tbl1.insert(tbl1.begin(), itGrp1->second.begin(), itGrp1->second.end());


        std::vector<ProtectedPtr<PhysicObject> > tbl2;
        tbl2.reserve(itGrp1->second.size());
        tbl2.insert(tbl2.begin(), itGrp2->second.begin(), itGrp2->second.end());

        for(std::vector<ProtectedPtr<PhysicObject> >::iterator it1 = tbl1.begin(); it1 != tbl1.end(); it1++)
        {
            for(std::vector<ProtectedPtr<PhysicObject> >::iterator it2 = tbl2.begin(); it2 != tbl2.end(); it2++)
            {
                if(it1->IsValid() && it2->IsValid()) // Not deleted yet
                {
                    PhysicObject *obj1 = it1->GetPtr();
                    PhysicObject *obj2 = it2->GetPtr();

                    // penetrationVector => Pour faire sortir *obj1 de *obj2
                    if(obj1->Intersect(*obj2, pos, penetrationVector))
                    {
                        obj1->OnIntersect(obj1, obj2, pos, penetrationVector);

                        // Might be deleted in the first event !
                        if(it1->IsValid() && it2->IsValid()) // Not deleted yet
                        {
                            obj2->OnIntersect(obj2, obj1, pos, -penetrationVector);
                        }
                    }
                }
            }
        }
    }
}
//------------------------------------------------------------------------------
/** Return the first unused physic group of this physic world */
int PhysicWorld::GetUnusedPhGroup() const
{
    if(m_phTbl.empty())
    {
        return 1;
    }
    else
    {
        std::map<int, std::vector<PhysicObject*> >::const_iterator it = m_phTbl.end();
        it--; // Last
        return it->first + 1;
    }
}
//------------------------------------------------------------------------------
/** Return the list of objects that collide the given physic group */
std::vector<CollisionInfo> PhysicWorld::IsIntersectWithGroup(const PhysicObject &obj, int phGroup) const
{

    std::map<int, std::vector<PhysicObject*> >::const_iterator phTblIt = m_phTbl.find(phGroup);
    std::vector<CollisionInfo> collisionTbl;

    if(phTblIt != m_phTbl.end())
    {
        const std::vector<PhysicObject*> &phTbl = phTblIt->second;

        Vector2D pos(0, 0), penetrationVector(0, 0);
        for(std::vector<PhysicObject*>::const_iterator it = phTbl.begin(); it != phTbl.end(); it++)
        {
            if(obj.Intersect(**it, pos, penetrationVector))
            {
                // penetrationVector => Pour faire sortir obj de **it
                collisionTbl.push_back(CollisionInfo(*it, pos, penetrationVector));
            }
        }
    }

    return collisionTbl;
}
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
} // namespace En
