-- Common functions used by guiLib

------------------------------------------------------------
-- Base fns
------------------------------------------------------------

function guiLib_private_destroyAttachedEvents(group)
	if group:UserDataExists("_eventToDestroyTbl") then
		local eventTbl = group:GetUserData("_eventToDestroyTbl")

		for i = 1, #eventTbl do
			eventTbl[i]:Destroy()
		end
		
		group:SetUserData("_eventToDestroyTbl", {})
	end
end

function guiLib_addAutoDestroyEvent(group, event)
	if event == nil then
		printErrorConsole("Error : guiLib_addAutoDestroyEvent => event == nil !")
		return
	end
	
	if group:GetPreDestroyFunction() == nil then
		group:SetPreDestroyFunction(guiLib_private_destroyAttachedEvents)
	end

	if not group:UserDataExists("_eventToDestroyTbl") then
		group:SetUserData("_eventToDestroyTbl", {})
	end
	
	local eventTbl = group:GetUserData("_eventToDestroyTbl")
	table.insert(eventTbl, event)
end

function guiLib_DestroyAttachedEventsRecursive(group)
	guiLib_private_destroyAttachedEvents(group)

	local childTbl = group:GetChildTable()
	
	for id, obj in pairs(childTbl) do
		local typeName = GetTypeName(obj:GetType())
		if (typeName == "Group") or (typeName == "Screen") then
			guiLib_DestroyAttachedEventsRecursive(obj)
		end
	end
end

function guiLib_removeAutoDestroyEvent(group, event)
	local isOk = false

	if (group:GetPreDestroyFunction() ~= nil) and (group:UserDataExists("_eventToDestroyTbl")) then -- Si existe
		local eventTbl = group:GetUserData("_eventToDestroyTbl")

		for i = 1, #eventTbl do
			if eventTbl[i] == event then
				table.remove(eventTbl, i)
				isOk = true
				break
			end
		end
	end
	
	if not isOk then
		printErrorConsole("Error : guiLib_removeAutoDestroyEvent => event not found !")
	end
end

------------------------------------------------------------
-- Get parent window
------------------------------------------------------------
function windowGetParentWindow(controlGroup)
	local group = controlGroup:GetParent()
	
	while true do
		if group:UserDataExists("C501_windowIdentifier") then
			return group
		elseif GetTypeName(group:GetType()) == "Screen" then -- Not in a window
			return nil
		end
	
		group = group:GetParent()
	end
end


------------------------------------------------------------
-- Return true if the point (a global position) is hidden by a toper window, false if not
------------------------------------------------------------
function windowIsPointHidden(windowToCompare, pointPos)
	if windowToCompare == nil then -- Not a window
		return false -- We asume the point is visible
	end

	local windowTbl = windowToCompare:GetParentScreen():GetUserData("windowTbl")

	-- We check if there is no other window on this window with a higher lowZpos
	for i = 1, #windowTbl do
		if windowTbl[i]:IsUpperThan(windowToCompare) and windowIsVisible(windowTbl[i]) then
			if windowTbl[i]:FindByNamePhRect("windowZone"):IsPointInside(pointPos) or (windowTbl[i]:GetUserData("isBorder") == true and windowTbl[i]:FindByNamePhRect("windowZoneBis"):IsPointInside(pointPos)) then
				return true -- A window is on top of this one !
			end
		end
	end
	
	-- We arrive here => We are the most higher window in which the user have clicked
	return false
end

------------------------------------------------------------
-- For the controls : return true if the Parent window is active, else otherwise
------------------------------------------------------------
function windowIsWinParentActive(controlGroup)
	local group = windowGetParentWindow(controlGroup)
	
	if group ~= nil then
		return windowIsActive(group)
	else -- Not in a window
		return true
	end
end
