
------------------------------------------------------------
-- Init
------------------------------------------------------------

function animCreate(spr, texTbl, stepTime)
	spr:SetUserData("_texTbl", texTbl)
	spr:SetUserData("_stepTime", stepTime)
	--spr:SetUserData("_flipped", false)
	spr:SetUserData("_timerEvt", nil)
	spr:SetUserData("_timerFn", nil)
	spr:SetUserData("_animTbl", nil)
end

------------------------------------------------------------
-- Set Anim
------------------------------------------------------------

function animChoseImageList(spr, animTbl)
	spr:SetUserData("_animTbl", animTbl)
	spr:SetUserData("_currentId", 1)
	animPrivateEnableTimer(spr, true)
	animPrivateTimerFunction(spr:GetUserData("_timerEvt"))
end

function animChoseOneImage(spr, imgId)
	animPrivateEnableTimer(spr, false) -- Pas de timer

	spr:SetUserData("_animTbl", {imgId})
	spr:SetUserData("_currentId", 1)
	
	local texTbl = spr:GetUserData("_texTbl")
	spr:SetImage(texTbl[imgId])
end


------------------------------------------------------------
-- Private
------------------------------------------------------------

function animPrivateEnableTimer(spr, enable)
	local evt = spr:GetUserData("_timerEvt")
	if enable and (evt == nil) then
		local stepTime = spr:GetUserData("_stepTime")
		local evt = spr:GetParentScreen():RegisterTimer(animPrivateTimerFunction, stepTime, true)
		guiLib_addAutoDestroyEvent(spr:GetParent(), evt)
		evt:SetUserData("_spr", spr)
		
		spr:SetUserData("_timerEvt", evt)
	elseif (not enable) and (evt ~= nil) then
		guiLib_removeAutoDestroyEvent(spr:GetParent(), evt)
		evt:Destroy()
		spr:SetUserData("_timerEvt", nil)
	end
end

function animPrivateTimerFunction(evt)
	local spr = evt:GetUserData("_spr")
	local animTbl = spr:GetUserData("_animTbl")
	
	if #animTbl >= 2 then
		local id = spr:GetUserData("_currentId")
		id = (id % #animTbl) + 1
		local texId = animTbl[id]
		spr:SetUserData("_currentId", id)
		
		local texTbl = spr:GetUserData("_texTbl")
		if (texId > 0) and (texId <= #texTbl) then
			spr:SetImage(texTbl[texId])
		else
			printErrorConsole("Error: bad texId (fn: animPrivateTimerFunction) !")
		end
	end
end

------------------------------------------------------------
-- Move anim
------------------------------------------------------------

function createMovingBlockAnim(grp, startPos, endPos, duration, isOverWay, finishedFn)
	-- Swap if needed
	if isOverWay then
		tmp = startPos
		startPos = endPos
		endPos = tmp
	end

	-- Create event
	local event = msgGrp:GetParentScreen():RegisterChronometer(privateMovingBlockAnimEvent, duration, true)
	event:SetUserData("isEntering", not isOverWay)
	event:SetUserData("startPos", startPos)
	event:SetUserData("endPos", endPos)
	event:SetUserData("grp", grp)
	guiLib_addAutoDestroyEvent(grp, event)
	
	
	-- Destroy any previous event
	if grp:UserDataExists("animEvent") then		
		local backEvent = grp:GetUserData("animEvent")
		if backEvent ~= nil then		
			-- We continue the movement => we get the previous ratio
			if event:GetUserData("isEntering") == backEvent:GetUserData("isEntering") then -- Same way
				event.ratio = backEvent.ratio
			else
				event.ratio = 1 - backEvent.ratio
			end
		
			guiLib_removeAutoDestroyEvent(grp, backEvent)
			backEvent:Destroy()
		end
	end
	
	-- Save event in the grp (to destroy it if needed)
	grp:SetUserData("animEvent", event)
	
	-- Finished function
	if type(finishedFn) == "function" then
		event:SetUserData("callbackEndFn", finishedFn)
	else
		event:SetUserData("callbackEndFn", nil)
	end
end

------------------------------------------------------------
-- Private
------------------------------------------------------------
-- Anim
function privateMovingBlockAnimEvent(ratio, event)
	-- Get vars
	local grp = event:GetUserData("grp")

	local endPos = event:GetUserData("endPos")
	local startPos = event:GetUserData("startPos")
	
	-- Acceleration ratio
	if event:GetUserData("isEntering") then
		ratio = ConvertRatioDeceleration(ratio, 9)
	else
		ratio = ConvertRatioAcceleration(ratio, 9)
	end
		
	-- Move
	grp:SetPosition(startPos - (startPos - endPos) * ratio)
	
	if ratio == 1 then	
		-- We destroy the event => it should not be accessed anymore
		grp:SetUserData("animEvent", nil)
		guiLib_removeAutoDestroyEvent(grp, event)
		event:Destroy()
		
		-- Callback function
		if event:UserDataExists("callbackEndFn") then
			event:GetUserData("callbackEndFn")(grp, event)
		end
	end
end
