------------------------------------------------------------
-- Button base
------------------------------------------------------------
function onPlayerButtonColision(playerPh, button, pos, vect)
	local player = playerPh:GetParent()
	
	if (isPlayer(player)) then
		player:SetUserData("_currentButton", button)
	
		if UserDataExistsAndIsNotEqual(button, "_btFnOnTouch", nil) then
			button:GetUserData("_btFnOnTouch")(player, button)
		end
	end
end


function initButtonFn(bt, fnOnTouch, fnOnAction)
	bt:SetUserData("_btFnOnTouch", fnOnTouch)
	bt:SetUserData("_btFnOnAction", fnOnAction)
end

------------------------------------------------------------
-- On / Off button
-- function : myFunction(player, status, button)
------------------------------------------------------------

function createOnOffButton(parentGrp, position, fn)
	local grp = parentGrp:CreateGroup("btOnOffGroup")
	grp:SetPosition(position)
	grp:SetUserData("status", false)
	grp:SetUserData("fn", fn)

	btSize = btOnTex:GetSize()

	local tmp = grp:CreateSprite("img", btOffTex, btSize / -2)
	tmp:SetSize(btSize)

	local ph = grp:CreatePhRect(phGroupButton, "ph", btSize / -2, btSize + Vector2D(0, 30))
	
	initButtonFn(ph, nil, function(player, button)
							local grp = button:GetParent()
	
							local status = not grp:GetUserData("status")
							
							onOffButtonSetSatus(grp, status)
							
							-- Notification
							local fn = grp:GetUserData("fn")
							if fn ~= nil then
								fn(player, status, grp)
							end
						end)

	-- Checkpoint
	grp:SetUserData("getStatusFn", function(grp) 
										return grp:GetUserData("status")
									end)
	grp:SetUserData("setStatusFn", function(grp, status)
										onOffButtonSetSatus(grp, status)
									end)
	registerSavedCheckpointObject(grp)

	-- Return
	return grp
end

function onOffButtonSetSatus(grp, status)
	grp:SetUserData("status", status)
	
	local spr = grp:FindByName("img")
	if status then
		spr:SetImage(btOnTex)
	else
		spr:SetImage(btOffTex)
	end
end

------------------------------------------------------------
-- Timer button
-- function : myFunction(player, status, button)
------------------------------------------------------------

function timerBtStep(evt)
	local grp = evt:GetUserData("grp")
	local step = grp:GetUserData("step") + 1
	local spr = grp:FindByName("img")
	
	if step <= btTimerNbSteps then
		spr:SetImage(btTimerStepTblTex[step])
	else
		guiLib_removeAutoDestroyEvent(grp, evt)
		evt:Destroy()
		grp:SetUserData("evt", nil)
	
		spr:SetImage(btTimerOff)
		
		local fn = grp:GetUserData("fn")
		if fn ~= nil then		
			fn(grp:GetUserData("playerOrigin"), false, grp)
		end
		
		-- Sound
		soundNs.PlaySnd(sndTimerBt, gameScreen)
	end
	
	grp:SetUserData("step", step)
end

function createTimerButton(parentGrp, position, duration, fn)
	local grp = parentGrp:CreateGroup("btTimerGroup")
	grp:SetPosition(position)
	grp:SetUserData("fn", fn)
	grp:SetUserData("duration", duration)

	btSize = btTimerOff:GetSize()

	local tmp = grp:CreateSprite("img", btTimerOff, btSize / -2)
	tmp:SetSize(btSize)

	local ph = grp:CreatePhRect(phGroupButton, "ph", btSize / -2, btSize + Vector2D(0, 30))
	
	initButtonFn(ph, nil, function(player, button)						
							local grp = button:GetParent()
							
							privTimerButtonStart(grp, playerOrigin)
							
							-- Notification
							local fn = grp:GetUserData("fn")
							if fn ~= nil then
								fn(player, true, grp)
							end
						end)
						
	-- Checkpoint
	grp:SetUserData("getStatusFn", function(grp) 
										local tbl = {}
										tbl.isOn = UserDataExistsAndIsNotEqual(grp, "evt", nil)
										
										if tbl.isOn then
											tbl.step = grp:GetUserData("step")
											tbl.stepElapsedTime = grp:GetUserData("evt"):GetElapsedTime()
										end
										
										return tbl
									end)
	grp:SetUserData("setStatusFn", function(grp, status)
										if status.isOn then
											privTimerButtonStart(grp, nil)
											grp:SetUserData("step", status.step - 1)
											timerBtStep(grp:GetUserData("evt"))
											grp:GetUserData("evt"):SetElapsedTime(status.stepElapsedTime)
										end
									end)
	registerSavedCheckpointObject(grp)

						
	-- Return
	return grp
end

function privTimerButtonStart(grp, playerOrigin)
	grp:SetUserData("step", 1)
	grp:SetUserData("playerOrigin", player)
	
	-- Destroy previous event:
	if UserDataExistsAndIsNotEqual(grp, "evt", nil) then
		local evt = grp:GetUserData("evt")
		guiLib_removeAutoDestroyEvent(grp, evt)
		evt:Destroy()
		grp:SetUserData("evt", nil)
	end
	
	local stepDuration = grp:GetUserData("duration") / btTimerNbSteps
	local evt = grp:GetParentScreen():RegisterTimer(timerBtStep, stepDuration, true)
	guiLib_addAutoDestroyEvent(grp, evt)
	evt:SetUserData("grp", grp)
	grp:SetUserData("evt", evt)
	
	local spr = grp:FindByName("img")
	spr:SetImage(btTimerStepTblTex[1])
end

------------------------------------------------------------
-- Door button
------------------------------------------------------------
function createDoorButton(door, button, isInvert)
	button:SetUserData("door", door)
	button:SetUserData("isInvert", isInvert)
	button:SetUserData("fn", function(player, status, grp)
								local door = grp:GetUserData("door")
								local isInvert = grp:GetUserData("isInvert")
								
								if isInvert then
									status = not status
								end									
								
								doorOpen(door, status, true)
							end)
end

------------------------------------------------------------
-- Removable area button
------------------------------------------------------------
function createRemovableAreaButton(removableAreaList, button, isInvert)
	button:SetUserData("removableAreaList", removableAreaList)
	button:SetUserData("isInvert", isInvert)
	button:SetUserData("fn", function(player, status, grp)
								local removableAreaList = grp:GetUserData("removableAreaList")
								local isInvert = grp:GetUserData("isInvert")
								
								if isInvert then
									status = not status
								end
								
								for i = 1, #removableAreaList do
									removableAreaEnable(removableAreaList[i], status, true)
								end
							end)
end
