------------------------------------------------------------
-- Level base
------------------------------------------------------------

levelGrp = nil
levelCheckpoint = nil
levelData = {}
levelLoadFn = nil
levelPlayerTbl = {}

function initLevelObjs() -- Load a level but to not create a checkpoint
	local levelScreen = gameScreen

	-- Remove players
	levelPlayerTbl = {}
	
	-- Remove sight
	local sightGrp = gameRenderScreen:FindByName("sightGroup")
	if sightGrp ~= nil then
		sightGrp:Destroy()
	end

	-- Doit rester une variable globale
	levelGrp = levelScreen:FindByName("levelGrp")
	
	-- Remove last level + the data added to the group
	if levelGrp ~= nil then
		levelGrp:Destroy()
	end
	levelGrp = levelScreen:CreateGroup("levelGrp")
	
	local infoGrp = gameRenderScreen:FindByName("infoMsgGrp")
	if infoGrp then
		infoGrp:SetName("destroye") -- Can't be found again
		infoGrp:Destroy()
	end
	
	-- Continue the stats using levelData
	levelStartContinueStats()
	
	-- Load new level
	levelLoadFn(levelGrp)
end

function levelStartContinueStats()
	
	if isLevelChallenge() then -- Challenge
		-- minTime est remis son ancienne valeur en cas de checkpoint
		levelGrp:SetUserData("getStatusFn", function(grp)
											return levelData.stats.minTime
										end)
		levelGrp:SetUserData("setStatusFn", function(grp, status)
											levelData.stats.minTime = status
										end)
		registerSavedCheckpointObject(levelGrp)
	else -- Normal
		-- minNbLaunch est remis son ancienne valeur en cas de mort !!!
		levelGrp:SetUserData("getStatusFn", function(grp)
											return levelData.stats.minNbLaunch
										end)
		levelGrp:SetUserData("setStatusFn", function(grp, status)
											levelData.stats.minNbLaunch = status
										end)
		registerSavedCheckpointObject(levelGrp)
	end
	
	-- Time
	local event = levelGrp:GetParentScreen():RegisterFrameEvent(function(frameTime, evt)
																	levelData.stats.minTime = levelData.stats.minTime + frameTime
																	
																	if isLevelChallenge() then
																		chronoCtrl.setValue(gameRenderScreen, levelData.stats.minTime)
																	end
																end)
	guiLib_addAutoDestroyEvent(levelGrp, event)
end

function levelInitStats()
	-- Level stats
	levelData.stats = {}
	levelData.stats.minTime = 0
	levelData.stats.minNbLaunch = 0
	levelData.stats.minDeath = 0
end

function loadLevel(name, gameMode, baseChallengeTime)
	if gameMode ~= "campagne" and gameMode ~= "challenge" then
		printConsoleError("Can't load level : gameMode '" .. gameMode .. "' unknown")
		return
	end

	-- Destroy any previous level
	destroyLevel()

	-- Camera zoom de base
	for id = 1, 2 do
		local target = gameRenderScreen:FindByName("screen" .. tostring(id) .. "grp"):FindByName("screen" .. tostring(id))
		local cfg = target:GetScreenConfig()
		cfg:CameraSetZoom(zoomBase)
		target:SetScreenConfig(cfg)
	end
	
	-- Music
	musicNs.loadPlayListLevel(name)

	-- The function used to load the level
	levelLoadFn = createCurrentLevel
	
	-- Load the level data
	initCurrentLevelData()
	
	-- Game mode
	levelData.gameMode = gameMode
	levelData.baseChallengeTime = baseChallengeTime

	-- Load a level but to not create a checkpoint
	initLevelObjs()
	
	-- Init stats
	levelInitStats()
	
	-- Chrono
	if isLevelChallenge() then
		chronoCtrl.create(gameRenderScreen)
		chronoCtrl.setValue(gameRenderScreen, levelData.stats.minTime)
	end
	
	-- Level name
	levelData.name = name
	
	-- Zoom
	adaptZoom(levelPlayerTbl)
	
	-- Make a checkpoint
	createCheckpoint()
end

function loadLevelFromFile(filename, gameMode, baseChallengeTime)
	levelLoadFn = nil

	dofile("scripts/levels/" .. filename)
	loadLevel(filename, gameMode, baseChallengeTime)
end

function destroyLevel()
	local levelScreen = gameScreen

	levelGrp = levelScreen:FindByName("levelGrp")
	
	-- Remove last level + the data added to the group
	if levelGrp ~= nil then
		levelGrp:Destroy()
	end
 
	levelGrp = nil
	levelCheckpoint = nil
	levelData = {}
	levelPlayerTbl = {}
	
	-- Sky back
	local tmp = gameRenderScreen:FindByName("skyBack")
	if tmp ~= nil then
		tmp:Destroy()
	end
	
	-- Msg grp
	local infoGrp = gameRenderScreen:FindByName("infoMsgGrp")
	if infoGrp then
		infoGrp:SetName("destroye") -- Can't be found again
		infoGrp:Destroy()
	end
	
	-- Sight
	local sightGrp = gameRenderScreen:FindByName("sightGroup")
	if sightGrp ~= nil then
		sightGrp:Destroy()
	end
	
	-- Chrono
	chronoCtrl.destroy(gameRenderScreen)
	
	-- Snd
	levelScreen:StopAllSounds()
	
	-- Garbadge collect
	collectgarbage()
end

function isLevelChallenge()
	return levelData.gameMode == "challenge"
end

function timeAddMalus(timeToAdd)
	levelData.stats.minTime = levelData.stats.minTime + timeToAdd
end

------------------------------------------------------------
-- Wall
------------------------------------------------------------

function createWall(grp, pos, size)
	return grp:CreatePhRect(phGroupDesign, "sol", pos, size)
end

------------------------------------------------------------
-- onPlayerKilled(reason)
------------------------------------------------------------
function onPlayerKilled(player, reason, data)
	-- View
	local waitTime = 1
	
	if reason == "bombeWall" then -- Explosion
		local bombeGrp = data
		CreateExplosion(bombeGrp:GetGlobalPosition(), math.pi / 2, bombeGrp:GetSpeed():GetDirection(), true)
		waitTime = 1
	else -- Player red
		player:FindByName("spr"):SetColor(Color(255, 0, 0))
		waitTime = 0.5
	end

	-- Stats
	levelData.stats.minDeath = levelData.stats.minDeath + 1

	-- Menu after wait time
	createInLevelMenu("dead", reason)
	guiLib_DestroyAttachedEventsRecursive(levelGrp)
	player:FindByName("body"):SetEventFunction(nil) -- No more collision
	WaitAndExec(function(safeGroup, waitTime, userData)										
					myChangeScreen(inLevelMenuScreen)
				end, waitTime, levelGrp, nil)
end

------------------------------------------------------------
-- Wall special
------------------------------------------------------------
function wallEnableRebound(wall, value, reboundForce)
	if value then
		if reboundForce == nil then
			reboundForce = 1
		end
	
		wall:SetUserData("_isRebound", true)
		wall:SetUserData("_reboundForce", reboundForce)
	elseif wall:UserDataExists("_isRebound") then
		wall:SetUserData("_isRebound", false)
	end
end

function wallEnableBombePass(wall, value)
	if value then
		wall:SetUserData("_isBombePass", true)
		wall:SetName("wallBombePass")
	elseif wall:UserDataExists("_isBombePass") then
		wall:SetUserData("_isBombePass", false)
		wall:SetName("wall")
	end
	
	-- Deco
	local tex = antiPlayerWallVTex
	local direction = Vector2D(0, 1)
	if wall:GetSize().x > wall:GetSize().y then -- horizontal
		tex = antiPlayerWallHTex
		direction = Vector2D(1, 0)
	end
	
	local tmp = wall:GetParent():CreateTexturedArea("movingTex", tex, wall:GetPosition(), wall:GetSize())
	tmp:SetTextureSpeed(direction * 20)
	tmp:SetZpos(10)
	tmp:SetScale(Vector2D(0.5, 0.5))
end

function wallEnablePlayerPass(wall, value)
	if value then
		wall:SetUserData("_isPlayerPass", true)
	elseif wall:UserDataExists("_isPlayerPass") then
		wall:SetUserData("_isPlayerPass", false)
	end
	
	-- Deco
	local tex = antiBombeWallVTex
	local direction = Vector2D(0, 1)
	if wall:GetSize().x > wall:GetSize().y then -- horizontal
		tex = antiBombeWallHTex
		direction = Vector2D(1, 0)
	end
	
	local tmp = wall:GetParent():CreateTexturedArea("movingTex", tex, wall:GetPosition(), wall:GetSize())
	tmp:SetTextureSpeed(direction * 20)
	tmp:SetZpos(10)
	tmp:SetScale(Vector2D(0.5, 0.5))
end

------------------------------------------------------------
-- Doors
------------------------------------------------------------

function doorCreate(parentGrp, position, size)
	local grp = parentGrp:CreateGroup("doorGrp", position)
	grp:SetZpos(10)
	
	-- Opened
	local spr = grp:CreateSprite("doorOpened", door1TexTbl["opened"], Vector2D(0, 0))
	local ratio = size.y / spr:GetSize().y
	spr:SetSize(spr:GetSize() * ratio) -- On garde seulement la hauteur
	
	-- Closed
	local spr = grp:CreateSprite("doorClosed", door1TexTbl["closed"], Vector2D(0, 0))
	spr:SetSize(size)

	--grp:CreatePhRect(phGroupDesign, "phDoor", Vector2D(0, 0), size)
	
	-- UserData
	grp:SetUserData("size", size)
	
	doorOpen(grp, false, false) -- Closed by default
	
	-- Checkpoint
	grp:SetUserData("getStatusFn", function(grp)
										return grp:FindByName("doorOpened"):IsVisible()
									end)
	grp:SetUserData("setStatusFn", function(grp, status)
										doorOpen(grp, status, false)
									end)
	registerSavedCheckpointObject(grp)
	
	-- Return
	return grp
end

function doorOpen(grp, isOpen, isView)
	grp:FindByName("doorOpened"):Show(isOpen)
	grp:FindByName("doorClosed"):Show(not isOpen)
	
	-- Ph
	local ph = grp:FindByName("phDoor")
	if ph == nil then
		if not isOpen then -- Need to be created
			grp:CreatePhRect(phGroupDesign, "phDoor", Vector2D(0, 0), grp:GetUserData("size"))
			forcePlayersGravity()
			
			if isView then
				soundNs.PlayGlobalSnd(sndDoorClose)
			end
		end
	else
		if isOpen then -- Need to be deleted
			ph:Destroy()
			forcePlayersGravity()
			
			if isView then
				soundNs.PlayGlobalSnd(sndDoorOpen)
			end			
		end
	end
end

------------------------------------------------------------
-- Hurt zone
------------------------------------------------------------

function createHurtZone(grp, pos, size, isFlip, isVertical)
	local ph = createWall(grp, pos, size)
	ph:SetUserData("_isHurtZone", true)
	--levelGrp:CreateShapeRectangle("hurtShape", ph:GetPosition(), ph:GetPosition() + ph:GetSize(), Color(255, 0, 0), 0, Color(0, 0, 0))
	
	-- Deco
	local tex = hurtPiquesVTex
	local scale = hurtPiquesVTex:GetSize().x / ph:GetSize().x
	if ((isVertical == nil) and (ph:GetSize().x > ph:GetSize().y) or (isVertical == false)) then -- horizontal
		tex = hurtPiquesHTex
		scale = hurtPiquesHTex:GetSize().y / ph:GetSize().y
		isVertical = false
	else
		isVertical = true
	end
	
	local tmp = ph:GetParent():CreateTexturedArea("movingTex", tex, ph:GetPosition(), ph:GetSize())
	tmp:SetScale(Vector2D(scale, scale))
	tmp:SetZpos(10)
	
	-- Flip
	if isFlip then
		if isVertical then
			tmp:FlipX(true)
		else
			tmp:FlipY(true)
		end
	end
	
	-- Return
	return ph
end

------------------------------------------------------------
-- Removable area
------------------------------------------------------------

function createRemovableArea(parentGrp, position, size, texture, centerOffset)
	local grp = parentGrp:CreateGroup("removableAreaGrp", position)
	
	-- Opened
	local spr = grp:CreateSprite("spr", texture, size / 2 - texture:GetSize() / 2 + centerOffset)
	
	-- UserData
	grp:SetUserData("size", size)
	
	removableAreaEnable(grp, false, false) -- removed by default
	
	-- Checkpoint
	grp:SetUserData("getStatusFn", function(grp)
										return grp:FindByName("ph") ~= nil
									end)
	grp:SetUserData("setStatusFn", function(grp, status)
										removableAreaEnable(grp, status, false)
									end)
	registerSavedCheckpointObject(grp)
	
	-- Return
	return grp
end

function removableAreaEnable(grp, isEnabled, isView)
	grp:FindByName("spr"):Show(isEnabled)
	
	-- Ph
	local ph = grp:FindByName("ph")
	if ph == nil then
		if isEnabled then -- Need to be created
			grp:CreatePhRect(phGroupDesign, "ph", Vector2D(0, 0), grp:GetUserData("size"))
			forcePlayersGravity()
		end
	else
		if not isEnabled then -- Need to be deleted
			ph:Destroy()
			forcePlayersGravity()
		end
	end
end

------------------------------------------------------------
-- Finish zone
------------------------------------------------------------

function privateOnPlayerExitColision(player, button)
	local isAllOnExit = true

	for i = 1, #levelPlayerTbl do
		if not levelPlayerTbl[i]:FindByName("body"):IsIntersect(button) then
			isAllOnExit = false
			break
		end
	end
	
	if isAllOnExit then
		createInLevelMenu("win", reason)
		myChangeScreen(inLevelMenuScreen)
	end
end


function createExitZone(parentGrp, position, size)
	-- Deco
	local grp = parentGrp:CreateGroup("levelExit")

	grp:SetPosition(position)
	
	local spr = grp:CreateSprite("spr", exitTex, Vector2D(0, 0))
	spr:SetSize(size)
	
	-- ph
	local ph = grp:CreatePhRect(phGroupButton, "exitPh", Vector2D(0, 0), size)
	initButtonFn(ph, privateOnPlayerExitColision, nil)
	
	-- Return
	return grp
end

------------------------------------------------------------
-- Sky
------------------------------------------------------------
function setSkyTexture(texture)
	local tmp = gameRenderScreen:FindByName("skyBack")
	if tmp ~= nil then
		tmp:Destroy()
	end

	local tmp = gameRenderScreen:CreateSprite("skyBack", texture, Vector2D(0, 0))
	tmp:SetZpos(-10000)
end

------------------------------------------------------------
-- Removable area
------------------------------------------------------------

function createRemovableArea(parentGrp, position, size, texture, centerOffset)
	local grp = parentGrp:CreateGroup("removableAreaGrp", position)
	
	-- Opened
	local spr = grp:CreateSprite("spr", texture, size / 2 - texture:GetSize() / 2 + centerOffset)
	
	-- UserData
	grp:SetUserData("size", size)
	
	removableAreaEnable(grp, false, false) -- removed by default
	
	-- Checkpoint
	grp:SetUserData("getStatusFn", function(grp)
										return grp:FindByName("ph") ~= nil
									end)
	grp:SetUserData("setStatusFn", function(grp, status)
										removableAreaEnable(grp, status, false)
									end)
	registerSavedCheckpointObject(grp)
	
	-- Return
	return grp
end

function removableAreaEnable(grp, isEnabled, isView)
	-- Ph
	local ph = grp:FindByName("ph")
	local changed = false
	
	if ph == nil then
		if isEnabled then -- Need to be created
			grp:CreatePhRect(phGroupDesign, "ph", Vector2D(0, 0), grp:GetUserData("size"))
			changed = true
		end
	else
		if not isEnabled then -- Need to be deleted
			ph:Destroy()
			forcePlayersGravity()
			changed = true
		end
	end
	
	if isView and changed then
		animBaseCreate(grp, function(grp, ratio, evt, userData)
								grp:FindByName("spr"):SetColor(Color(255, 255, 255, 255 * ratio))
							end, 0.3, false, not isEnabled, false, nil)
		soundNs.PlaySnd(sndRemovableWall, gameScreen)
	else
		if isEnabled then
			grp:FindByName("spr"):SetColor(Color(255, 255, 255, 255))
		else
			grp:FindByName("spr"):SetColor(Color(255, 255, 255, 0))
		end		
	end
end

------------------------------------------------------------
-- Simple image
------------------------------------------------------------

function createSimpleImage(parentGrp, tex, position, size, isInFront)
	-- Deco
	local spr = parentGrp:CreateSprite("simpleImg", tex, position)
	
	if size ~= nil then
		spr:SetSize(size)
	end
	
	if isInFront then
		spr:SetZpos(100)
	end
	-- Return
	return spr
end

------------------------------------------------------------
-- Msg zone
------------------------------------------------------------

function createMessageZone(parentGrp, position, size, text, duration)
	-- Ph
	createMessageZoneMitraille(parentGrp, position, size, {text}, duration)
end

function createMessageZoneMitraille(parentGrp, position, size, textTbl, duration)
	if duration == nil then
		duration = 2
	end

	local grp = parentGrp:CreateGroup("msgZoneGrp", position)
	grp:SetUserData("isAlreadyDisplayed", false) -- We create only one checkpoint per zone !
	grp:SetUserData("textTbl", textTbl)
	grp:SetUserData("duration", duration)

	-- Ph
	local ph = grp:CreatePhRect(phGroupButton, "ph", Vector2D(0, 0), size)
	
	-- Fn
	initButtonFn(ph, function(player, button)
						local grp = button:GetParent()
						if UserDataExistsAndIsNotEqual(grp, "isAlreadyDisplayed", true) then
							grp:SetUserData("tlbId", 1)
							grp:SetUserData("isAlreadyDisplayed", true)
							privateShowMsg(button:GetParent())
						end
					end, nil)
end

function privateShowMsg(grp)
	local isAlreadyCreated = grp:GetUserData("isAlreadyDisplayed")
	local tlbId = grp:GetUserData("tlbId")

	local textTbl = grp:GetUserData("textTbl")
	local duration = grp:GetUserData("duration")
	createInfoMsg(gameRenderScreen, textTbl[tlbId], duration)
	soundNs.PlaySnd(sndCheckpoint, gameScreen)
	
	if tlbId ~= #textTbl then
		grp:SetUserData("tlbId", tlbId + 1)
		WaitAndExec(privateShowMsg, duration + 3, grp, nil)
	end
end
