userCfg = userCfg or {}

------------------------------------------------------------
-- Conf
------------------------------------------------------------
userCfg.keyConf = {}
userCfg.soundConf = {}

------------------------------------------------------------
-- Load
------------------------------------------------------------

function userCfg.LoadAllConf()
	userCfg.LoadKeyConf()
	userCfg.LoadSoundConf()
end

function userCfg.LoadKeyConf()
	local node = GetConfigManager():GetRootConfigNode():AccessNode("options"):AccessNode("control")
	
	userCfg.keyConf = userCfg.AddUnsetInTable(node:GetLuaTable(), userCfg.GetDefaultKeyConf())
	--userCfg.keyConf = userCfg.GetDefaultKeyConf()
end

function userCfg.LoadSoundConf()
	local node = GetConfigManager():GetRootConfigNode():AccessNode("options"):AccessNode("sound")
	
	userCfg.soundConf = userCfg.AddUnsetInTable(node:GetLuaTable(), userCfg.GetDefaultSoundConf())
end

------------------------------------------------------------
-- Save
------------------------------------------------------------

function userCfg.SaveAllConf()
	userCfg.SaveKeyConf()
	userCfg.SaveSoundConf()
end

function userCfg.SaveKeyConf()
	local node = GetConfigManager():GetRootConfigNode():AccessNode("options"):AccessNode("control")
	
	node:Clear()
	node:SetLuaTable(userCfg.AddUnsetInTable(userCfg.keyConf, userCfg.GetDefaultKeyConf()))
end

function userCfg.SaveSoundConf()
	local node = GetConfigManager():GetRootConfigNode():AccessNode("options"):AccessNode("sound")
	
	node:Clear()
	node:SetLuaTable(userCfg.AddUnsetInTable(userCfg.soundConf, userCfg.GetDefaultSoundConf()))
end

------------------------------------------------------------
-- Create the option window
------------------------------------------------------------

function userCfg.GetDefaultSoundConf()
	return {musicVolume=50, soundVolume=100}
end

function userCfg.GetDefaultKeyConf()
	local tbl = {}
	
	tbl.player2Keys = {}
	tbl.player2Keys["keyRight"] = "Right"
	tbl.player2Keys["keyLeft"] = "Left"
	tbl.player2Keys["keyJump"] = "Up"
	tbl.player2Keys["keyAction"] = "Down"


	-- Player
	tbl.player1Keys = {}
	tbl.player1Keys["keyRight"] = "D"
	tbl.player1Keys["keyLeft"] = "Q"
	tbl.player1Keys["keyJump"] = "Z"
	tbl.player1Keys["keyAction"] = "S"

	-- fakeMouse
	tbl.isFakeMouse = false -- Can be redefined
	tbl.fakeMouseSpeed = 200
	tbl.fakeMouseKeys = {}
	tbl.fakeMouseKeys["keyRight"] = "L"
	tbl.fakeMouseKeys["keyLeft"] = "J"
	tbl.fakeMouseKeys["keyUp"] = "I"
	tbl.fakeMouseKeys["keyDown"] = "K"
	tbl.fakeMouseKeys["keyLaunch"] = "Space"
	
	return tbl
end

function userCfg.AddUnsetInTable(tbl, valuesToAdd)
	-- Add unset values
	for key, value in pairs(valuesToAdd) do
		if tbl[key] == nil then
			tbl[key] = value
		elseif type(valuesToAdd[key]) == "table" then
			userCfg.AddUnsetInTable(tbl[key], value)
		end
	end
	
	-- Remove keys that are only present in tbl
	for key, value in pairs(tbl) do
		if valuesToAdd[key] == nil then
			tbl[key] = nil
		end
	end
	
	-- Return
	return tbl
end
