------------------------------------------------------------
-- Zoom
------------------------------------------------------------

function onPlayerMove(player)	
	local id = player:GetUserData("playerId")
	local screenRenderer = gameRenderScreen:FindByName("screen" .. tostring(id) .. "grp"):FindByName("screen" .. tostring(id))
	
	setScreenRendererCenter(screenRenderer, viewGetCameraCenter(player))
end

function viewGetCameraCenter(player)
	local id = player:GetUserData("playerId")
	local screenRenderer = gameRenderScreen:FindByName("screen" .. tostring(id) .. "grp"):FindByName("screen" .. tostring(id))
	local decalage = nil
	
	if id == 1 then
		decalage = Vector2D(screenSize.x / 4 / screenRenderer:GetScreenConfig():CameraGetZoom().x, 0)
	else
		decalage = Vector2D(-1 * screenSize.x / 4 / screenRenderer:GetScreenConfig():CameraGetZoom().x, 0)
	end
	
	local lookDecal = player:GetUserData("lookDecal")
	
	return player:GetGlobalPosition() - gameScreen:GetPosition() + lookDecal + decalage + viseurCenterPos
end

function setScreenRendererCenter(renderer, center)
	cfg = renderer:GetScreenConfig()
	cfg:CameraSetCenter(center)
	renderer:SetScreenConfig(cfg)
end



------------------------------------------------------------
-- Zoom
------------------------------------------------------------

function adaptZoom(playerTbl)
	for i = 1, #playerTbl do
		local player = playerTbl[i]
		local id = player:GetUserData("playerId")
		local screenRenderer = gameRenderScreen:FindByName("screen" .. tostring(id) .. "grp"):FindByName("screen" .. tostring(id))
	
		if player:GetUserData("mode") == "move" then
			createZoomAnim(player, screenRenderer, moveZoom, zoomAnimTime)
		elseif player:GetUserData("mode") == "throw" or player:GetUserData("mode") == "look" then
			createZoomAnim(player, screenRenderer, throwZoom, zoomAnimTime)
		end
	end
end

function createZoomAnim(player, screenRenderer, targetZoom, animTime)
	local event = gameScreen:RegisterChronometer(function(ratio, event)
			ratio = ConvertRatioCosAccelerationDeceleration(ratio)
			
			local cfg = event:GetUserData("target"):GetScreenConfig()
			cfg:CameraSetZoom(event:GetUserData("start") + event:GetUserData("mvt") * ratio)
			event:GetUserData("target"):SetScreenConfig(cfg)
		
			-- Camera pos
			local player = event:GetUserData("player")
			onPlayerMove(player)
		
			if ratio == 1 then
				guiLib_removeAutoDestroyEvent(player, event)
				event:Destroy()
			end
		end, animTime, true)
	
	event:SetUserData("player", player)
	event:SetUserData("start", screenRenderer:GetScreenConfig():CameraGetZoom().x)
	event:SetUserData("mvt", targetZoom - event:GetUserData("start"))
	event:SetUserData("target", screenRenderer)
	guiLib_addAutoDestroyEvent(player, event)
end

function createCameraMoveAnim(screenRenderer, targetPos)
	local data = {}
	data["target"] = screenRenderer
	data["startPos"] = screenRenderer:GetScreenConfig():CameraGetCenter()
	data["endPos"] = targetPos

	animBaseCreate(levelGrp, function(grp, ratio, evt, userData)
			ratio = ConvertRatioCosAccelerationDeceleration(ratio)			
			setScreenRendererCenter(userData["target"], userData["startPos"] + (userData["endPos"] - userData["startPos"]) * ratio)
		end, 0.4, false, false, true, data)
end

function viewCameraReturnToPlayer(player)
	local targetPos = viewGetCameraCenter(player) - player:GetUserData("lookDecal")

	local id = player:GetUserData("playerId")
	local screenRenderer = gameRenderScreen:FindByName("screen" .. tostring(id) .. "grp"):FindByName("screen" .. tostring(id))

	createCameraMoveAnim(screenRenderer, targetPos)
end

------------------------------------------------------------
-- Walk
------------------------------------------------------------

function viewPlayerChangeMode(player, newMode)		
	if newMode == "move" then
		animChoseOneImage(player:FindByNameSprite("spr"), 5)
	elseif newMode == "look" then
		animChoseOneImage(player:FindByNameSprite("spr"), 1)
	elseif newMode == "throw" then
		animChoseOneImage(player:FindByNameSprite("spr"), 2)
	end
end

function viewPlayerWalk(walk, player, directionX)
	local spr = player:FindByNameSprite("spr")
	
	if walk then
		spr:FlipX(directionX < 0)
	end
	
	if UserDataExistsAndIsNotEqual(player, "_lastWalkState", walk) then -- Changement d'état
		if walk then
			animChoseImageList(spr, {4, 5})
		else
			animChoseOneImage(spr, 5)
		end	
	end
	player:SetUserData("_lastWalkState", walk)
end

function viewPlayerSight(player, size, direction, speed)
	-- Player
	local spr = player:FindByNameSprite("spr")
	spr:FlipX(direction.x < 0)
	
	-- Limité
	local isLimited = (size >= bombeSightMaxDist * 0.99)
	
	-- Put the cursor
	local spr = gameRenderScreen:FindByName("sightGroup"):FindByName("cursorSpr")
	
	if isLimited then
		spr:SetImage(cursorSightEndTex)
		spr:SetCenter(Vector2D(spr:GetSize().x / 2, 4))
		spr:SetColor(Color(255, 0, 0))
	else
		spr:SetImage(cursorSightTex)
		spr:SetCenter(Vector2D(spr:GetSize().x / 2, 0))
		local ratio = size / bombeSightMaxDist
		spr:SetColor(Color(55 + 200 * ratio, 255 - 90 * ratio, 0))
	end
	
	spr:SetPosition(direction * size)
	spr:SetRotation(direction:GetAngle() - math.pi / 2)
	
	--[[if UserDataExistsAndIsNotEqual(player, "sightLine", nil) then
		player:GetUserData("sightLine"):Destroy()
		player:SetUserData("sightLine", nil)
	end
	
	player:SetUserData("sightLine", player:CreateShapeLine("sightLine", viseurCenterPos, direction * size * (1 / throwZoom) + viseurCenterPos, 4, Color(0, 255, 0), 0, Color(0, 0, 0)))]]
end
