//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "glUtils.hpp"

#include <iostream>

#include "MyException.hpp"
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
using namespace std;
//------------------------------------------------------------------------------
std::list<sf::FloatRect> GlCut::m_cutList;
//------------------------------------------------------------------------------
GlCut::GlCut(const sf::FloatRect &rect, sf::RenderTarget &App)
{
    // View rect coords
    sf::FloatRect viewRect = App.GetView().GetRect();

    // Transform local coordonates to window coordonates
    sf::FloatRect windowRect;
    windowRect.Left = (rect.Left - viewRect.Left) / viewRect.GetWidth() * App.GetWidth();
    windowRect.Top = (rect.Top - viewRect.Top) / viewRect.GetHeight() * App.GetHeight();
    windowRect.Right = (rect.Right - viewRect.Left) / viewRect.GetWidth() * App.GetWidth();
    windowRect.Bottom = (rect.Bottom - viewRect.Top) / viewRect.GetHeight() * App.GetHeight();

    // Final rect
    // Change Y origine and active GL cut
    sf::FloatRect finalRect(windowRect.Left, App.GetHeight() - windowRect.Bottom, windowRect.Right, App.GetHeight() - windowRect.Top);

    // Active GL cut
    if(m_cutList.empty()) // Si pas encore activé
    {
        glEnable(GL_SCISSOR_TEST);
    }
    else
    {
        sf::FloatRect overlappingRect(0, 0, 0, 0);

        if(finalRect.Intersects(m_cutList.back(), &overlappingRect)) // On active que si ce glCut faitpartie du précédent
        {
            finalRect = overlappingRect;
        }
        else
        {
           finalRect = sf::FloatRect(0, 0, 0, 0); // Rien ne passe
        }
    }

    // Save to list
    glScissorSfRect(finalRect);
    m_cutList.push_back(finalRect);
}
//------------------------------------------------------------------------------
void GlCut::glScissorSfRect(const sf::FloatRect &rect)
{
    glScissor(rect.Left, rect.Top, rect.GetWidth(), rect.GetHeight());
}
//------------------------------------------------------------------------------
GlCut::~GlCut()
{
    m_cutList.pop_back();

    if(m_cutList.empty()) // Plus rien dans la liste => on arrete là
    {
        glDisable(GL_SCISSOR_TEST);
    }
    else // On active le cut précedent
    {
        glScissorSfRect(m_cutList.back());
    }
}
//------------------------------------------------------------------------------
} // namespace En
